;;
;; cchelp.el
;;
;; Help with Chinese characters
;;
;; by Stephen G. Simpson <simpson@math.psu.edu>
;; with apologies to the author of gb2pinyin.el,
;; D.-P. Deng <deng@bunny.rhic.bnl.gov>.
;;
;; NOTE: This is the EMACS-19 version of cchelp.el,
;; for use with cemacs.el and Release 19 of GNU Emacs.
;; 
;; 31 October 1993 -- original version
;; 19 November 1993 -- added cchelp-pop-buffer
;;    (thanks to HIROSE Tutomu <muri@foretune.co.jp>)
;;

;;
;; Put the following in your ~/.emacs or default.el or cemacs.el.
;; 
;  (autoload 'cchelp "cchelp"
;    "Give helpful information about the Chinese character at point.\n" t)
;;
;  (define-key help-map "o" 'cchelp)
;;

(defvar cchelp-file "/usr/local/lib/emacs/zidian.gb.sorted"
  "*Sorted help file for Chinese characters.\n")

(defvar cchelp-pop-buffer nil
  "*This variable influences the behavior of cchelp.
If non-nil, the CCHelp buffer is shown whenever cchelp is invoked.
Default is nil.\n")

(defvar cchelp-window-height 8 "*Height of the CCHelp window.
Effective only when cchelp-pop-buffer is non-nil.\n")

;;
;; To override a defvar, do a setq in your ~/.emacs.  For example,
;; I am using Big5 instead of GB, so my ~simpson/.emacs contains
;;
;  (setq cchelp-file "/usr/local/lib/emacs/zidian.b5.sorted")
;;
;; Your ~/.emacs might contain
;;
;  (setq cchelp-file (expand-file-name "~/helpfile.gb))
;;

(let ((process-connection-type nil)) 
  (start-process "cchelp" "CCHelp" "mlook" cchelp-file))

(set-process-filter (get-process "cchelp") 'cchelp-filter)

(defun cchelp ()
  "Give helpful information on the Chinese character at point.
The help information accumulates in the CCHelp buffer.\n"
  (interactive)
  (if (and (<= (+ (point) 2) (point-max))
	   (>= (following-char) 160))
      (process-send-string "cchelp"
	(concat (char-to-string (following-char))
		(char-to-string (char-after (1+ (point))))
		"\n"))
    (message "not a Chinese character")))

(defun cchelp-filter (proc str)
  (let* ((msg (substring str 0 -1))
	 (buf (process-buffer proc))
	 (win (get-buffer-window buf))
	 old-buf old-win)
    (if (and cchelp-pop-buffer (not win))
	(progn
	  (split-window-vertically cchelp-window-height)
	  (switch-to-buffer buf)
	  (setq win (selected-window))
	  (other-window 1)))
    (setq old-buf (current-buffer))
    (setq old-win (selected-window))
    (unwind-protect
	(progn
	  (set-buffer buf)
	  (if win (select-window win))
	  (goto-char (point-max))
	  (insert (concat "\n" msg))
	  (if (not win) (message msg)))
      (set-buffer old-buf)
      (select-window old-win))))

;;
;; end of cchelp.el (EMACS-19 version)
;;
