;;
;; cchelp.el
;;
;; Help with Chinese characters
;;
;; by Stephen G. Simpson <simpson@math.psu.edu>,
;; with apologies to the author of gb2pinyin,
;; D.-P. Deng <deng@bunny.rhic.bnl.gov>.
;;
;; NOTE: This is the MULE version of cchelp.el, for use with
;; Mule, the Multilingual Emacs.
;;
;; 11 September 1993 -- initial version
;; 27 October 1993 -- changed cchelp key to help-map "o"
;; 1 November 1993 -- added SiSheng support
;; 19 Nov 1993 -- added cchelp-pop-buffer
;;   (thanks to HIROSE Tutomu <muri@foretune.co.jp>)
;; 

;;
;; Put the following in your ~/.emacs or mule/lisp/default.el
;; 
;  (autoload 'cchelp "cchelp"
;    "Give helpful information on the Chinese character at point.\n" t)
;;
;  (define-key help-map "o" 'cchelp)
;;
;; (setq x-mouse-bind-hook '(lambda ()
;;   (define-key mouse-map x-button-c-left '(lambda ()
;      (x-mouse-set-point) (cchelp)))))
;; 
;; or, if you have x-sb-mouse,
;;
;  (setq x-mouse-bind-hook '(lambda ()
;    (x-mouse-define-key "x-mouse-c1-window-click" t 'default '(lambda ()
;      (x-mouse-set-point) (cchelp)))))
;;

(defvar cchelp-gb-file "/usr/local/lib/emacs/zidian.gb.sorted"
  "*Sorted help file for GB-coded Chinese characters.\n")

(defvar cchelp-b5-file "/usr/local/lib/emacs/zidian.b5.sorted"
  "*Sorted help file for Big5-coded Chinese characters.\n")

(defvar cchelp-py2ss nil "*This variable influences the behavior of cchelp.
Set this to t if you like SiSheng fonts, nil otherwise.
Default is nil.\n")

(defvar cchelp-gb-buffer "CCHelp")

(defvar cchelp-b5-buffer "CCHelp")

(defvar cchelp-pop-buffer nil
  "*This variable influences the behavior of cchelp.
If non-nil, cchelp shows the cchelp buffer whenever it is invoked.
Default is nil.\n")

(defvar cchelp-window-height 8 "*Height of cchelp window.
Effective only when cchelp-pop-buffer is non-nil.\n")

;;
;; To override a defvar, put an appropriate setq in your ~/.emacs
;; or mule/lisp/default.el.  For example, your ~/.emacs might contain
;;
;  (setq cchelp-gb-file (expand-file-name "~/helpfile.gb"))
;;
;  (setq cchelp-py2ss t)
;;
;  (setq cchelp-pop-buffer t)
;;

(if cchelp-py2ss (require 'py2ss))

(defvar cchelp-gb-process nil)
(defvar cchelp-b5-process nil)

(defun cchelp-start-gb ()
  (setq cchelp-gb-process
	(start-process "cchelp-gb" cchelp-gb-buffer
		       "mlook" cchelp-gb-file))
  (set-process-coding-system cchelp-gb-process *euc-china* *euc-china*)
  (set-process-filter cchelp-gb-process 'cchelp-filter))

(defun cchelp-start-b5 ()
  (setq cchelp-b5-process
	(start-process "cchelp-b5" cchelp-b5-buffer
		       "mlook" cchelp-b5-file))
  (set-process-coding-system cchelp-b5-process *big5-eten* *big5-eten*)
  (set-process-filter cchelp-b5-process 'cchelp-filter))

(defun cchelp ()
  "Give helpful information on the Chinese character at point.\n
The information accumulates in cchelp-gb-buffer and cchelp-b5-buffer.\n"
  (interactive)
  (cond
   ((looking-at "\\cc")
    (or (and cchelp-gb-process
	     (equal (process-status cchelp-gb-process) 'run))
	(cchelp-start-gb))
    (process-send-string
     cchelp-gb-process
     (concat (char-to-string (following-char)) "\n")))
   ((looking-at "\\ct")
    (or (and cchelp-b5-process
	     (equal (process-status cchelp-b5-process) 'run))
	(cchelp-start-b5))
    (process-send-string
     cchelp-b5-process
     (concat (char-to-string (following-char)) "\n")))
   (t (message "not a Chinese character"))))

(defun cchelp-filter (proc str)
  (let* ((msg
	  (if cchelp-py2ss 
	      (py2ss-string (substring str 0 -1))
	    (substring str 0 -1)))
	 (buf (process-buffer proc))
	 (win (get-buffer-window buf))
	 old-buf old-win)
    (if (and cchelp-pop-buffer (not win))
	(progn
	  (split-window-vertically cchelp-window-height)
	  (switch-to-buffer buf)
	  (setq win (selected-window))
	  (other-window 1)))
    (setq old-buf (current-buffer))
    (setq old-win (selected-window))
    (unwind-protect
	(progn
	  (set-buffer buf)
	  (if win (select-window win))
	  (goto-char (point-max))
	  (insert (concat "\n" msg))
	  (if (not win) (message msg)))
      (set-buffer old-buf)
      (select-window old-win))))

;; comment this out if using same buffer for GB and Big5
;;(let ((old-buf (current-buffer)))
;;  (unwind-protect
;;      (progn
;;	(set-buffer (process-buffer cchelp-gb-process))
;;	(set-file-coding-system *euc-china*)
;;	(set-buffer (process-buffer cchelp-b5-process))
;;	(set-file-coding-system *big5-eten*))
;;    (set-buffer old-buf)))

;;
;; end of cchelp.el (Mule version)
;;

