;; py2ss.el
;; Functions for converting ascii PinYing to SiSheng font string.
;;
;; suggestion and bug report to muri@foretune.co.jp
;; 93/10/18 muri


(defconst py2ss-consonant "[bpmfdtnlgkhjqxzcsrzyw]")
(defconst py2ss-vowel "[aeiouvmn:]+")
(defconst py2ss-tail "[ngr]*")
(defconst py2ss-tone "[1-5]")
(defconst py2ss-pat
      (concat "\\(" py2ss-consonant "*\\)"
	      "\\(" py2ss-vowel "\\)"
	      "\\(" py2ss-tail "\\)"
	      "\\(" py2ss-tone "\\)"))

(defconst py2ss-vow-alist
  '((?a 1 ("a" "" "" "" ""))
    (?e 2 ("e" "" "" "" ""))
    (?i 3 ("i" "" "" "" ""))
    (?o 4 ("o" "" "" "" ""))
    (?u 5 ("u" "" "" "" ""))
    (?v 6 ("" "" "" "" ""))
    (?m 7 ("m" "m" "" "m" "m"))
    (?n 8 ("n" "n" "" "" ""))
    ))

(defun py2ss-string (str)
  "Convert pinying string STR, return sisheng string."
  (interactive)
  (let ((result "") ss)
    (while (string-match py2ss-pat str)
      (let ((beg (match-beginning 0))
	    (end (match-end 0)))
	(setq py (substring str beg end))
	(setq ss (py2ss-word py))
	(setq result
	      (concat result
		      (if (> beg 0) (substring str 0 beg))
		      (or ss py)))
	(setq str (substring str end))))
    (concat result str)))

(defun py2ss-umlaut (str)
  (if (string-match "\\(.*\\)\\(u\\)\\(.*\\)\\(:\\)\\(.*\\)" str)
      (concat (substring str (match-beginning 1) (match-end 1))
	      "v"
	      (substring str (match-beginning 3) (match-end 3))
	      (substring str (match-beginning 5) (match-end 5)))
    str))

(defun py2ss-word (str)
  (setq str (py2ss-umlaut (downcase str)))
  (if (not (string-match py2ss-pat str))
      nil
    (let* ((con (substring str (match-beginning 1) (match-end 1)))
	   (vow (substring str (match-beginning 2) (match-end 2)))
	   (tail (substring str (match-beginning 3) (match-end 3)))
	   (tone (- (string-to-char
		     (substring str (match-beginning 4) (match-end 4)))
		    ?0))
	   (vow-len (length vow))
	   (i 0)
	   (last-al nil)
	   al last-i c)
      (while (< i vow-len)
	(setq c (aref vow i))
	(setq al (assoc c py2ss-vow-alist))
	(if (and al
		 (or (null last-al)
		     (< (car (cdr al))
			(car (cdr last-al)))))
	    (progn
	      (setq last-al al)
	      (setq last-i i)))
	(setq i (1+ i)))
      (if (> tone 4)
	  (setq tone 0))
      (setq i 0)
      (setq new-vow "")
      (while (< i vow-len)
	(setq new-vow
	      (concat new-vow
		      (if (/= i last-i)
			  (char-to-string (aref vow i))
			(nth tone (car (cdr (cdr last-al)))))))
	(setq i (1+ i)))
      (concat con new-vow tail))))

(provide 'py2ss)
